<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
        <meta name="description" content="SATHub Index Page">
        <meta name="viewport" content="width=device-width">
        <link rel="icon" type="image/png" href="{{ url_for('static', filename='favicon.png') }}">
        <title>SATHub {{ product.version }}</title>
    </head>
    <body>
        <h1>Projeto SATHub {{ product.version }}</h1>
        <p>Servidor <a href="https://github.com/base4sistemas/sathub/">SATHub</a> está em execução<br/>
            <em>Copyright &copy; Base4 Sistemas Ltda. All rights reserved.</em>
            </p>
    </body>
</html>